'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _uniq = require('lodash/uniq');

var _uniq2 = _interopRequireDefault(_uniq);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

var _Error = require('./Error');

var _jsBeautify = require('js-beautify');

var _htmlMinifier = require('html-minifier');

var _mjml = require('./helpers/mjml');

var _defaultStyle = require('./defaultStyle');

var _defaultStyle2 = _interopRequireDefault(_defaultStyle);

var _documentParser = require('./documentParser');

var _documentParser2 = _interopRequireDefault(_documentParser);

var _dom = require('./helpers/dom');

var _dom2 = _interopRequireDefault(_dom);

var _MJMLElementsCollection = require('./MJMLElementsCollection');

var _MJMLElementsCollection2 = _interopRequireDefault(_MJMLElementsCollection);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _server2 = _interopRequireDefault(_server);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('mjml-engine/mjml2html');

var mjml2html = function mjml2html(content) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var parsedContent = content;

  if (typeof content == 'string') {
    debug('Start parsing content');
    parsedContent = (0, _documentParser2.default)(content);
    debug('Content parsed.');
  }

  debug('Start rendering');
  var html = render({ mjml: parsedContent, options: options });
  debug('Done rendering');

  return html;
};

var removeCDATA = function removeCDATA(str) {
  return str.replace(/<!--\[CDATA\[(.*?)\]\]-->/g, '$1');
};

var insertColumnMediaQuery = function insertColumnMediaQuery($) {
  var mediaQueries = [];

  (0, _each2.default)({ 'mj-column-per': '%', 'mj-column-px': 'px' }, function (unit, className) {
    var columnWidths = [];

    $('[class*="' + className + '"]').each(function () {
      columnWidths.push($(this).data('column-width'));
      $(this).removeAttr('data-column-width');
    });

    (0, _uniq2.default)(columnWidths).forEach(function (width) {
      var mediaQueryClass = className + '-' + width;

      mediaQueries.push('.' + mediaQueryClass + ', * [aria-labelledby="' + mediaQueryClass + '"] { width:' + width + unit + '!important; }');
    });
  });

  if (mediaQueries.length > 0) {
    var mediaQuery = '<style type="text/css">\n      @media only screen and (min-width:480px) {\n        ' + mediaQueries.join('\n') + '\n      }\n      </style>';

    $('head').append(mediaQuery);
  }

  return $;
};

var fixOutlookLayout = function fixOutlookLayout($) {
  var bodyWidth = $('.mj-body').data('width');

  $('.mj-body').removeAttr('data-width');

  $('.mj-section-outlook-background').each(function () {
    var url = $(this).data('url');
    var width = $(this).data('width');

    $(this).removeAttr('class').removeAttr('data-url').removeAttr('data-width');

    if (!url) {
      return;
    }

    $(this).before('<!--[if gte mso 9]>\n      <v:rect xmlns:v="urn:schemas-microsoft-com:vml" fill="true" stroke="false" style="width:' + width + 'px;">\n        <v:fill origin="0.5, 0" position="0.5,0" type="tile" src="' + url + '" />\n        <v:textbox style="mso-fit-shape-to-text:true" inset="0,0,0,0">\n      <![endif]-->');

    $(this).after('<!--[if gte mso 9]>\n        </v:textbox>\n      </v:rect>\n      <![endif]-->');
  });

  $('.mj-body-outlook-open').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      <table border="0" cellpadding="0" cellspacing="0" width="' + bodyWidth + '" align="center" style="width:' + bodyWidth + 'px;"><tr><td>\n      <![endif]-->');
  });

  $('.mj-body-outlook-line').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      </td></tr></table>\n      <![endif]-->\n      <!--[if mso]>\n      <table border="0" cellpadding="0" cellspacing="0" width="' + bodyWidth + '" align="center" style="width:' + bodyWidth + 'px;"><tr><td>\n      <![endif]-->');
  });

  $('.mj-body-outlook-close').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      </td></tr></table>\n      <![endif]-->');
  });

  $('.mj-section-outlook-open').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      <table border="0" cellpadding="0" cellspacing="0"><tr><td style="width:' + parseInt($(this).data('width')) + 'px;">\n      <![endif]-->');
  });

  $('.mj-section-outlook-line').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      </td><td style="width:' + parseInt($(this).data('width')) + 'px;">\n      <![endif]-->');
  });

  $('.mj-section-outlook-close').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      </td></tr></table>\n      <![endif]-->');
  });

  $('.mj-social-outlook-open').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      <table border="0" cellpadding="0" cellspacing="0" align="center"><tr><td>\n      <![endif]-->');
  });

  $('.mj-social-outlook-line').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      </td><td>\n      <![endif]-->');
  });

  $('.mj-social-outlook-close').each(function () {
    $(this).replaceWith('<!--[if mso]>\n      </td></tr></table>\n      <![endif]-->');
  });

  $('.mj-divider-outlook').each(function () {
    var insertNode = '<table border="0" cellpadding="0" cellspacing="0" style="' + $(this).attr('style') + '"></table>';

    $(this).removeClass('mj-divider-outlook').after('<!--[if mso]>' + insertNode + '<![endif]-->');
  });

  return $;
};

var fixLegacyAttrs = function fixLegacyAttrs($) {
  var legacyAttrs = ['align', 'valign', 'bgcolor', 'border', 'background'];

  $('#YIELD_MJML').css({ background: $('.mj-body').data('background-color') });
  $('.mj-body').removeAttr('data-background-color');

  // https://github.com/facebook/react/issues/140 ...
  // server side workaround to allow custom tags.
  legacyAttrs.forEach(function (attr) {
    var dataAttr = 'data-legacy-' + attr;

    $('[' + dataAttr + ']').each(function () {
      $(this).attr(attr, $(this).attr(dataAttr));
      $(this).removeAttr(dataAttr);
    });
  });

  return $;
};

var clean = function clean($) {
  $('#YIELD_MJML').each(function () {
    $(this).removeAttr('id');

    if ($(this).attr('style') === '') {
      $(this).removeAttr('style');
    }
  });

  $('.mj-body').removeAttr('class').removeAttr('style');

  $('.mj-content').removeAttr('class');

  return $;
};

var container = function container() {
  var title = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
  var content = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];
  return '<!doctype html>\n<html xmlns="http://www.w3.org/1999/xhtml">\n<head>\n<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>\n<title>' + title + '</title>\n<style type="text/css">' + _defaultStyle2.default + '</style>\n<!--[if !mso]><!-->\n<style type="text/css">\n  @import url(https://fonts.googleapis.com/css?family=Ubuntu:400,500,700,300);\n</style>\n<style type="text/css">\n  @media only screen and (max-width:480px) {\n    @-ms-viewport { width:320px; }\n    @viewport { width:320px; }\n  }\n</style>\n<link href="https://fonts.googleapis.com/css?family=Ubuntu:400,500,700,300" rel="stylesheet" type="text/css">\n<!--<![endif]-->\n</head>\n<body id="YIELD_MJML">\n  ' + content + '\n</body>\n</html>';
};

var render = function render(_ref) {
  var mjml = _ref.mjml;
  var options = _ref.options;

  var content = '';

  if (mjml) {
    debug('Create root React element');
    var rootElemComponent = _react2.default.createElement(_MJMLElementsCollection2.default[mjml.tagName.substr(3)], { mjml: (0, _mjml.parseInstance)(mjml) });

    debug('Render to static markup');
    content = _server2.default.renderToStaticMarkup(rootElemComponent);
  } else {
    throw new _Error.EmptyMJMLError('.render: No MJML to render in options ' + options.toString());
  }

  debug('React rendering done. Continue with special overrides.');
  var $ = _dom2.default.parseHTML(container(options.title, content));

  $('.mj-raw').each(function () {
    $(this).replaceWith($(this).html());
  });

  $ = insertColumnMediaQuery($);
  $ = fixLegacyAttrs($);
  $ = fixOutlookLayout($);
  $ = clean($);

  var html = _dom2.default.getHTML($);

  html = removeCDATA(html);

  if (options.beautify && _jsBeautify.html) {
    html = (0, _jsBeautify.html)(html, {
      indent_size: 2,
      wrap_attributes_indent_size: 2
    });
  }

  if (options.minify && _htmlMinifier.minify) {
    html = (0, _htmlMinifier.minify)(html, {
      collapseWhitespace: true,
      removeEmptyAttributes: true,
      minifyCSS: true
    });
  }

  return html;
};

exports.default = mjml2html;
//# sourceMappingURL=mjml2html.js.map
